// Dynamic link library implementation of NeuroSolutions BackGammaAxon component 

#include "NSDLL.h"

/********************************/
/* Backpropagation of component */

__declspec(dllexport) void performBackGammaAxon(
	DLLData *instance,	// Pointer to instance data (may be NULL)
	DLLData *dualInstance,	// Pointer to the forward axons instance data (may be NULL)
	NSFloat	*error,	 	// Pointer to the current error vector
	int 	rows,		// Number of rows of PEs in the layer
	int 	cols,		// Number of columns of PEs in the layer
	NSFloat	*delayedError, 	// Pointer to the error vector delayed by a user defined time step
	int 	taps,		// Number of memory taps specified by the user in the inspector
	NSFloat	*data,	 	// Pointer to the layers of processing elements (PEs)
	NSFloat	*gamma,		// Pointer to a vector of gamma coefficients, one for each channel
	NSFloat	*gradient 	// Pointer to the gamma gradient vector
	)
{
	register int i,j,k,length=rows*cols;

	for (i=0; i<length; i++)
		for (j=1; j<taps; j++) {
			k = i + j*length;
			error[k-length] += gamma[i]*delayedError[k];
			error[k] += (1.0f-gamma[i])*delayedError[k];
			if (gradient) 
				gradient[i] += delayedError[k]*(data[k-length]-data[k]);
		}
}

/******************************************/
/* Management of instance data (OPTIONAL) */
/*
__declspec(dllexport) DLLData *allocBackGammaAxon(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	DLLData	*dualInstance,	// Pointer to forward axons instance data (may be NULL)
	int 	rows,		// Number of rows of PEs in the layer
	int 	cols,		// Number of columns of PEs in the layer
	int 	taps		// Number of taps attached to each PE
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	return instance;
}

__declspec(dllexport) void freeBackGammaAxon(DLLData *instance)
{
	freeDLLInstance(instance);
}
*/